#!/usr/bin/env python3
# -fire CLI
from fire import Fire

from tdb_io.version import __version__

import pandas as pd

from influxdb import InfluxDBClient
#from pymongo import MongoClient
#import pymongo # to get ASCENDING
import datetime as dt
import time
import os
import json
import sys

import socket
import sys

# DEBUG=1  # will change in MAIN  args
ACTUAL_CREDENTIALS={}


# SSL warning
import warnings
warnings.filterwarnings('ignore', message='Unverified HTTPS request')


# if DEBUG:print("D...  project/module:  tdb_io/influx :", __version__ )


#OK
#===============================================================
def is_int(n):
    print(f"/{n}/")
    try:
        float_n = float(n)
        print(f"/{float_n}/")
        int_n = int(float_n)
        print(f"/{int_n}/")
    except ValueError:
        return False
    else:
        return float_n == int_n


def check_port(IP="127.0.0.1"):
    """
Checks if influx runs on IP
    """
    ok = False
    client = InfluxDBClient(host=IP, port=8086)
    try:
        client.get_list_database()
        ok = True
    except Exception as ex:
        # print(ex)
        if type(ex).__name__.find("ConnectionError")==0:
            print("X... (chkport) )NO DATABASE ON ",IP)
        if  type(ex).__name__.find("InfluxDBClientError")==0:
            print("i... database exists, authorization needed")
            ok = True
    return ok



def check_databases(IP="127.0.0.1", user="", password=""):
    """
Checks if influx runs on IP AND shows databases
r=client.query('SELECT "temp" FROM "autogen"."idx232"')

    """
    #if DEBUG:print("D... checkd: ",IP,user)
    ok = False
    autho = False
    dbs = []
    if user=="":
        client = InfluxDBClient(host=IP, port=8086)
    else:
        client = InfluxDBClient(host=IP, port=8086, username=user, password=password)
    try:
        dbs = client.get_list_database()
        ok = True
    except Exception as ex:
        # print(ex)
        if type(ex).__name__.find("ConnectionError")==0:
            print("X... (chkdb) NO DATABASE ON ",IP)
        if  type(ex).__name__.find("InfluxDBClientError")==0:
            print("i... database exists, authorization needed")
            ok = False
            autho = True
    if autho and user=="":
        print("I... trying with credentials from CONFIG")
        creds,ips = read_infl_credentials( DEBUG = 0)

        # NO SSL
        gotit = False
        try:
            #if DEBUG:print("D... chkd:  getting client NO SSL")
            client = InfluxDBClient(IP, 8086,creds[0],creds[1],creds[2],ssl=False, timeout=3)
            client.get_list_database() # this checks error
            gotit = True
        except:
            print("X... exception without SSL, trying with SSL")

        if not gotit:
            try:
                #if DEBUG:print("D... chkd: getting client WITH SSL")
                client = InfluxDBClient(IP, 8086,creds[0],creds[1],creds[2],ssl=True, timeout=3)
                client.get_list_database() # this checks error
            except:
                print("X... ",IP, 8086,creds[0],creds[1],creds[2] )

        dbs = client.get_list_database()
    return dbs





def check_series(database="", series="",
                 qlimit=5,
                 IP="127.0.0.1",
                 user="", password="",
                 delete=False):
    """
Checks if influx runs on IP, fir each DATABASE it shows ALL SERIES
r=client.query('SELECT "temp" FROM "autogen"."idx232"')

    """
    #if DEBUG:print("D... checks: ",IP,user)

    ok = False
    autho = False
    dbs = []
    liseries = []
    if user=="":
        client = InfluxDBClient(host=IP, port=8086)
    else:
        client = InfluxDBClient(host=IP, port=8086, username=user, password=password)
    try:
        dbs = client.get_list_database()
        ok = True
    except Exception as ex:
        # print(ex)
        if type(ex).__name__.find("ConnectionError")==0:
            print("X... (chkser) NO DATABASE ON ",IP)
        if  type(ex).__name__.find("InfluxDBClientError")==0:
            print("i... database exists, authorization needed")
            ok = False
            autho = True
    if autho and user=="":
        print("I... trying with credentials from CONFIG")
        creds,ips = read_infl_credentials( DEBUG=0)

        # NO SSL
        gotit = False
        limit_int = True
        mytimeout =3 # below 10000 events

        # ------------ make time possible  1h 1d 1w
        #  > SELECT "water_level" FROM "h2o_feet" WHERE time > now() - 1h
        if type(qlimit) is str:
            limit_int = False
            if not qlimit[-1] in ["s","m","h","d","w"]:
                print("X...   limit is string but it is not 1s 1m 1h 1d 1w:", qlimit)
                print("X.... exit")
                sys.exit(1)
            intnum = "".join(qlimit[:-1])
            if  not is_int(intnum):
                print("X...   limit is string but it is not  integer s m h d w:", qlimit, intnum)
                print("X.... exit")
                sys.exit(1)
            mytimeout = 30 # worst case scenario
        #-------------
        if limit_int:
            if qlimit > 10000:
                mytimeout = 10
            if qlimit > 60000:
                mytimeout = 20
            if qlimit > 120000:
                mytimeout = 30


        try:
            client = InfluxDBClient(IP, 8086,creds[0],creds[1],creds[2],ssl=False, timeout=mytimeout)
            dbs = client.get_list_database() # generates exception
            gotit = True
        except:
            print("X... chks: exception without SSL")

        if not gotit:
            #if DEBUG:print("D... chks: trying with SSL")
            client = InfluxDBClient(IP, 8086,creds[0],creds[1],creds[2],ssl=True, timeout=mytimeout)
            dbs = client.get_list_database() # generates excep

        #if DEBUG:print("D... client obtained - nossl")
        dbs = client.get_list_database()
    #print("D... list of databases obtained", dbs)


    TERMW = 55
    TERMW, rows = os.get_terminal_size(0)
    #TERMW-=1
    #-------------------------------- if series not given, list them
    if series == "":
        #if DEBUG:print("D... stage check....")
        for i in dbs:
            if i["name"].find("_")==0:
                continue
            if (database!="")and(database!=i["name"]):
                continue
            print("\n",i['name'],"_"*(TERMW-len(i['name'])-2) )
            sers = client.get_list_series( i['name'] )
            # print(sers)

            maxlen=1
            for j in sers:
                #print(j,len(j))
                if len(j)>maxlen:
                    maxlen=len(j)
            maxlen+=2
            one = int(TERMW/maxlen)

            #print("D... "*50, maxlen, one )
            k = 0
            for j in sers:
                k+= 1
                liseries.append(j)
                print( "{ss:{maxlens}}".format(ss=j, maxlens=maxlen), end="" )
                if k % one == 0:
                    print()
            print()
        print()
        return liseries

    #-----series given....we go to "read" option
    #if DEBUG:print("D... stage read....")
    dbs = [i['name'] for i in dbs]
    if not database in dbs:
        print("X ... no such database - exit", database,"in", dbs)
        sys.exit(1)# quit()
    i = database
    print(i,"_"*(TERMW-len(i)-2) )

    sers = client.get_list_series( i )
    if not series in sers:
        print("X ... no such series found - exit", series)
        sys.exit(1)# quit()

    client.switch_database(i)
    #
    # i must find columns:

    #r = client.query('SELECT "temp" FROM "autogen"."'+series+'" ')

    # print("________________________________")
    # if limit_int:
    #     print("i... quering INC with limit: ", qlimit)
    #     r = client.query('SELECT * FROM "autogen"."'+series+'" ORDER BY time ASC LIMIT '+str(qlimit)  )
    # else:
    #     print("i... quering INC with time: ", qlimit)
    #     r = client.query('SELECT * FROM "autogen"."'+series+'" WHERE TIME > now() - '+str(qlimit) +" ORDER BY time ASC " )

    # #print(r.raw)
    # cols = r.raw['series'][0]['columns']
    # points = r.get_points()
    # print(cols)
    # print()
    # ppoints=0
    # dfdict = {}
    # divme = 1000
    # #h5 = h5py.File("savetest.h5")
    # for p in points:
    #     #print(p, type(p))
    #     dfdict[ppoints] = p
    #     ppoints+=1
    #     if (ppoints % divme)==0:
    #         print("{:9.0f} * {}  ".format(ppoints/ divme, divme ), end="\r" )
    # print("\nCOLUMNS:",cols)
    # print("TOTAL: {} points printed, the demanded number was {}".format( ppoints, qlimit ) )
    # df = pd.DataFrame.from_dict(dfdict, "index")
    # print(df.head() )
    # print(df.tail() )

    print("________________________________")
    if limit_int:
        print("i... quering DESC with limit: ", qlimit)
        r = client.query('SELECT * FROM "autogen"."'+series+'" ORDER BY time DESC LIMIT '+str(qlimit)  )
    else:
        print("i... quering DESC with time: ", qlimit)
        r = client.query('SELECT * FROM "autogen"."'+series+'" WHERE TIME > now() - '+str(qlimit) +" ORDER BY time DESC " )

    # r = client.query('SELECT * FROM "autogen"."'+series+'" ORDER BY time DESC LIMIT '+str(qlimit)  )

    #print(r.raw)
    cols = r.raw['series'][0]['columns']
    points = r.get_points()
    print(cols)
    print()
    ppoints=0
    dfdict = {}
    divme = 1000
    #h5 = h5py.File("savetest.h5")
    for p in points:
        #print(p, type(p))
        dfdict[ppoints] = p
        ppoints+=1
        if (ppoints % divme)==0:
            print("{:9.0f} * {}  ".format(ppoints/ divme, divme ), end="\r" )
    print("\nCOLUMNS:",cols)
    print("TOTAL: {} points printed, the DEMANDED number was {}".format( ppoints, qlimit )," - latest first" )

    df = pd.DataFrame.from_dict(dfdict, "index")
    #print(df.head() )
    #print(df.tail() )
    print(df)

    #hdf = HDFStore("influx_{}_{}.h5".format(series,ppoints) )
    now = dt.datetime.now().strftime("%Y%m%d_%H%M%S")
    fname = "influx_{}_{}_{}".format(series,now,ppoints)
    fnameh5 = "{}.h5".format(fname)
    fnamecsv = "{}.csv".format(fname)
    fnameparq = "{}.parquet".format(fname)

    # tables NEEDE HERE
    if 'tables' in globals():
        print("i... saving h5", fnameh5)
        df.to_hdf( fnameh5, series ,format ="table", mode = 'a' ) # this was ment as incremental .... sorry
    else:
        print("i... NOT saving h5", fnameh5, "format=table nedds tables: doesnt work in windows")

    print("i... saving csv", fnamecsv)
    df.to_csv( fnamecsv )
    print("i... saving parquet", fnameparq)
    df.to_parquet( fnameparq )

    if delete:
        DELETE = 'DELETE  FROM "'+series+'" '
        print( DELETE )
        res = input("REALLY DELETE? y/n")
        if res == "y":
            r = client.query( DELETE)

    print()
    return






def _read_series(database="test", series='idx0', IP="127.0.0.1", user="", password="", delete = False):
    """
READS ONE SERIES - MOST COMPELETE FUNCTION HERE
r=client.query('SELECT "temp" FROM "autogen"."idx232"')

    """
    ok = False
    autho = False
    dbs = []
    if user=="":
        client = InfluxDBClient(host=IP, port=8086)
    else:
        client = InfluxDBClient(host=IP, port=8086, username=user, password=password)
    try:
        dbs = client.get_list_database()
        ok = True
    except Exception as ex:
        # print(ex)
        if type(ex).__name__.find("ConnectionError")==0:
            print("X... (rdser) NO DATABASE ON ",IP)
        if  type(ex).__name__.find("InfluxDBClientError")==0:
            print("i... database exists, authorization needed")
            ok = False
            autho = True
    if autho and user=="":
        print("I... trying with credentials from CONFIG")
        creds,ips = read_infl_credentials( DEBUG=0)
        # NO SSL
        client = InfluxDBClient(IP, 8086,creds[0],creds[1],creds[2],ssl=False, timeout=3)
        #if DEBUG:print("D... client obtained - empty")
        dbs = client.get_list_database()
    dbs = [i['name'] for i in dbs]
    #if DEBUG:print("D... list of databases obtained", dbs)


    TERMW = 55
    TERMW, rows = os.get_terminal_size(0)

    if not database in dbs:
        print("X ... no such database - exit", database,"in", dbs)
        sys.exit(1)#quit()
    i = database
    print(i,"_"*(TERMW-len(i)-2) )

    sers = client.get_list_series( i )
    if not series in sers:
        print("X ... no such series found - exit", series)
        sys.exit(1)# quit()

    client.switch_database(i)
    #
    # i must find columns:
    #

    #r = client.query('SELECT "temp" FROM "autogen"."'+series+'" ')
    r = client.query('SELECT * FROM "autogen"."'+series+'" ')
    #print(r.raw)
    cols = r.raw['series'][0]['columns']
    points = r.get_points()
    print(cols)
    for p in points:
        print(p)
    print("\nCOLUMNS:",cols)

    if delete:
        DELETE = 'DELETE  FROM "'+series+'" '
        print( DELETE )
        res = input("REALLY DELETE? y/n")
        if res == "y":
            r = client.query( DELETE)

    return
    return




def checkout_hostname_database(database="test", series='idx0', IP="127.0.0.1", user="", password="", delete = False):
    """
    Check Hostname in the LOCAL influx DATABASES --- IF DIFFER: DROP;
    """

    MYHOSTNAME = socket.gethostname()
    NEWDB = "i_am_" + MYHOSTNAME
    #if DEBUG:print("D... gethostname:", MYHOSTNAME, NEWDB )

    IP = "127.0.0.1"
    ok = False
    autho = False
    dbs = []

    if user=="":
        client = InfluxDBClient(host=IP, port=8086)
    else:
        client = InfluxDBClient(host=IP, port=8086, username=user, password=password)
    try:
        dbs = client.get_list_database()
        ok = True
    except Exception as ex:
        if type(ex).__name__.find("ConnectionError")==0:
            print("X... (chhodb) NO DATABASE ON ",IP)
        if  type(ex).__name__.find("InfluxDBClientError")==0:
            print("i... database exists, authorization needed")
            ok = False
            autho = True
    if autho and user=="":
        print("I... trying with credentials from CONFIG")
        creds,ips = read_infl_credentials(DEBUG=0)

        # NO SSL
        gotit = False
        try:
            client = InfluxDBClient(IP, 8086,creds[0],creds[1],creds[2],ssl=False, timeout=3)
            dbs = client.get_list_database()
            gotit = True
            #if DEBUG:print("D... client obtained with NO SSL")
        except:
            print("X... NO SSL - didnt work, I try SSL")

        if not gotit:
            try:
                #if DEBUG:print("D... client trying with  SSL")
                client = InfluxDBClient(IP, 8086,creds[0],creds[1],creds[2],ssl=True, timeout=3)
                dbs = client.get_list_database()
                #if DEBUG:print("D... client obtained with WITH SSL")
            except:
                print("X... WITH SSL - didnt work", IP, 8086,creds[0],creds[1],creds[2])


        dbs = client.get_list_database()
    dbs = [i['name'] for i in dbs]

    #if DEBUG:print("D... list of databases obtained", dbs)

    host_present = False
    ohost_present = False
    ohost_name = "x"
    for i in dbs:
        if i == NEWDB:
            host_present = True
        elif i.find("i_am_")==0:
            ohost_present = True
            ohost_name = i
    if ohost_present:
        print("!... other host present:",ohost_name," - DROP it:", ohost_present)
        client.drop_database(ohost_name)
        #return
    if host_present:
        #if DEBUG:print("D... all ok, my hostname is there with i_am_...")
        return
    print("!... CREATE DATABASE myself:", NEWDB)
    client.create_database(NEWDB)


#def drop_measurement():





def read_infl_credentials(config="~/.influx_userpassdb", DEBUG=True):
    """ READ and RETURN Influxdb  Credentials
    """
    ips=[]
    ips1=[]
    ips2=[]
    filemis = False
    try:
        with open( os.path.expanduser("~/.seread_discover8086") ) as f:
            ips1=f.readlines()
    except:
        filemis = True
        if DEBUG:
            print("X... NO FILE ~/.seread_discover8086 with automatic IPs")

    try:
        with open( os.path.expanduser("~/.seread_permanent8086") ) as f:
            ips2=f.readlines()
        filemis = False
    except:
        filemix = True
        if DEBUG:
            print("X... NO FILE ~/.seread_permanent8086 with permanent IPs")

    if filemis: print("X... no .seread_permanent8086 NEITHER seread_discover8086 with servers!")

    ips=ips1+ips2
    ips=[ i.strip() for i in ips]
    #================ credentials HERE============
    try:
        with open(os.path.expanduser( config ) ) as f:
            creds=f.readlines()
        creds=[ i.strip() for i in creds ]
    except:
        print("X... no credentials in",config )
        sys.exit(1)
        return (["","","test"],ips)
    if DEBUG:print("D... exiting with",creds,ips)
    return (creds,ips)






#=================================================

def influxwrite( DATABASE="test", MEASUREMENT="test",
                 values="" ,
                 IP="127.0.0.1",
                 initialconfig="2DOlater" , DEBUG = False):
    """write data to influx. DATABASE=test,MEASUREMENT=hostname; ./influx.py influxwrite -values b=1
    """
    if DEBUG:print("D... ************************ writin to influx ******************************")
    creds,ips=read_infl_credentials( DEBUG=DEBUG)
    # print( creds ,"##", ips )

    for IP in ips:
        if DEBUG:print("D... IP=",IP)
        if len(IP)<7:
            print("X... IP too short......... skip")
            continue
        if IP[0].isdigit(): #
            print("i... IP is number......... no ssl used")
            client = InfluxDBClient(IP, 8086,creds[0],creds[1],creds[2],ssl=False, timeout=3)
        else:
            if DEBUG:print("D... IP is text. ...........I do ssl ")
            client = InfluxDBClient(IP, 8086,creds[0],creds[1],creds[2],ssl=True, verify_ssl=False,timeout=8)

        MYHOSTNAME=socket.gethostname()
        if DEBUG:print("D....................MYHOST=",MYHOSTNAME )


        if DEBUG:print("D....................INFLUX WRITING= DB/MEASUREMENT",DATABASE,MEASUREMENT )

        if len(values)==0:
            if DEBUG:print("D....  no values given ... do you need to look at current values? -exit")
            #mongo2h5( DATABASE, COLLECTION, nlimit=100000, write_h5=False)
            #quit()
            sys.exit(1)
        insertvals=values.split(",")
        if DEBUG:print("D... insert pairs:", insertvals)
        INSERTDICT={}
        for i in insertvals:
            if DEBUG:print("D... insert pair:",i)
            key=i.split("=")[0]
            val=float(i.split("=")[1])
            INSERTDICT[key]=val
        if DEBUG:print("D... INSERT DICT:", INSERTDICT )

        ###json_body = [ {"measurement":MYHOSTNAME+"_test4"} ]
        json_body = [ {"measurement":MYHOSTNAME+"_"+MEASUREMENT} ]
        #json_body[0]["fields"]={}
        json_body[0]["fields"]=INSERTDICT
        if DEBUG:print("D... ",json_body)
        res = client.write_points(json_body)
        #print("writepoints response=",res)
        if DEBUG:print("D... ************************ influx DONE ******************************")
        return res # client write response






#=================================================


#=============================================================
#=============================================================
#=============================================================
if __name__=="__main__":


    print("F... you can run ")
    print('  ./influx.py influxwrite test testmes "a=1" --DEBUG  ')
    print()
    Fire( {
      'influxwrite':influxwrite ,
      'check_port':check_port ,
      'check_dbs':check_databases ,
      'check_series':check_series ,
#      'read_series':read_series ,
#      'help': help
  } )



#!/usr/bin/env python3
# -fire CLI
from fire import Fire
from tdb_io.version import __version__




from pymongo import MongoClient
import pymongo # to get ASCENDING
import datetime
import time

from pandas.plotting import register_matplotlib_converters
register_matplotlib_converters()



#  from monary import Monary #  10x faster BUT CANNOT FIND mongoc.h
import numpy
import numpy as np
import pandas as pd

import argparse
import os
import json

import matplotlib.pyplot as plt
import matplotlib.dates as mdates

#================== I import from myself........:)
#import influx

DEBUG=1  # will change in MAIN  args
ACTUAL_CREDENTIALS={}
#=================================================

print("D...  project/module:  tdb_io/mongo :", __version__ )
#print("i... module tdb_io/mongo is loaded")


#=================================================

def read_mongo_credentials( configfile="~/.mymongo.local" , silent=False):
    """Reads filename with credentials:  ~/.mymongo.local   Returns: output DICT with credentials
    """
    cfgfile=os.path.expanduser( configfile )
    if DEBUG:print("D... reading credentials", cfgfile)
    if not silent:print("i... reading ", cfgfile , end="\n" )
    cdict={}
    if os.path.isfile( cfgfile ):
        with open( cfgfile ,'r') as f:
            cdict=json.load( f )
        if not silent:print( ' ... HOST=',cdict['host'], end="\n" )
    else:
        if not silent:print(" !... NO FILE FOUND")
        if not silent:print("""
X...   try to create some JSON file like:
____________________________________________
{
"host":"127.0.0.1",
"username":"user",
"password":"xxx",
"authSource":"admin"
}
____________________________________________
        """)
        quit()
    print("i... credentioals ..                             [OK]")
    return cdict  # returns credentials============



#=================================================


def test_read_mongo_credentials():
    print("D... test function ... run pytest")
    r=read_mongo_credentials()
    assert  len(r["host"])>0
    assert  len(r["username"])>0
    assert  len(r["password"])>0
    assert  len(r["authSource"])>0





#=================================================


# from reading file, gives an extra question, returns client
#   ~/.mymongo.local   assumed   TEST and DATA

def get_mongo_database( database , initialconfig="~/.mymongo.local" , silent=False, return_none=True  ):
    """
    Return: client_to_database / needs databasename
    """
    global ACTUAL_CREDENTIALS
    if database=="":
        print("X... no database given, try test, data, mysql ...")
        return None
    print(ACTUAL_CREDENTIALS)
    if  len(ACTUAL_CREDENTIALS)==0:
        print("W... no actual credentials, reading the file",initialconfig)
        ACTUAL_CREDENTIALS=read_mongo_credentials( initialconfig ,silent=silent)
    print("D...",ACTUAL_CREDENTIALS)
    if DEBUG:print("D... has read credentials")
    if DEBUG: print( "D... ", ACTUAL_CREDENTIALS)
    if ACTUAL_CREDENTIALS['host']=="":
        print("X.... host not defined in cfg, bad cfg file...") # rare case
        quit()
    client = MongoClient( ACTUAL_CREDENTIALS['host'],
                     username=ACTUAL_CREDENTIALS['username'],
                     password=ACTUAL_CREDENTIALS['password'],
                     authSource=ACTUAL_CREDENTIALS['authSource'],
#                    authMechanism='SCRAM-SHA-1', # !!! later versions
                     serverSelectionTimeoutMS=1000,
                     connect=False
     )
   #  { "host":"127.0.0.1",
   # "username":"ojr",
   # "password":"aaaa",
   # "authSource":"data"
   #     }

    ###---- ONE EXTRA question == slowdown, but no messing later
    q=False
    if DEBUG:print("D... extra slowingdown question on ismaster")
    try:
        if DEBUG:print("D... asking admin: ismaster: ... ")
        result = client.admin.command("ismaster")
        if DEBUG:print("D... client is                                   [OK]")

    except:
    #except errors.ConnectionFailure:
        print("!... Connection failed <- ismaster check")
        q=True
    if q:
        try:
             errors
        except:
            print("!... `errors` object not found ... mongo database doesnt exist?")
            print("!...  may authSource is not 'auth' but 'data' ? ")
            print("!... quitting");quit()
    #not authorizes!!!  print( client.database_names() )
    print("D... NAMES==",    client.database_names() )
    if return_none:
        print("D...    you   asked  to   return   NONE.... returning NONE")
        return None
    else:
        return client[ database ]
    print("D...  get_mongo_database ...........             [OK]")





#=================================================
#                "~/.mymongo.local"
def tell_mongo_collections( database , initialconfig="~/.mymongo.local" ):
    """
    lists collections in the database . NEEDS:  databasename /test/data/
    """
    dbase=get_mongo_database( database ,initialconfig, silent=True, return_none=False)
    if DEBUG:print("i... dbase obtained. Searching collections names:", database )
    collection = dbase.collection_names(include_system_collections=False)
    #print( "{:10s}   {}".format( database,sorted(collection) ) )
    print( "\n=== {:10s} ==========================".format( database ) )
    print(   " collection name      | number of records".format( database ) )
    print(   "_________________________________________" )
    for i in sorted(collection):
        print("{:>19s}  ".format(i) , end="" )
        a=[]
        la=dbase[ i ].count()
        #la=list(a)
        print("   {:9d}".format(  la )   )
    #print("D.... returning collection")
    return collection





#=================================================

def get_date_from_mark( atime ,string=False ): # 1d 1h 1w
    """ Returns START time. NEEDS: number+time_unit ... 5m 30m 1h 3d 5w
        returns START time
              Either  timedelta
              Or      string (if mongodb t is string)
    """
    unit=1
    if atime[-1]=='S': unit=1
    if atime[-1]=='M': unit=60
    if atime[-1]=='H': unit=60*60
    if atime[-1]=='d': unit=60*60*24
    if atime[-1]=='m': unit=60*60*24*30
    if atime[-1]=='y': unit=3600*24*365
    if atime[-1]=='w': unit=60*60*24*7
    val=atime[:-1]
    ret=float(val) * unit
    print( "D... time distance:  value=",val," unit=", unit ," ... seconds=", ret )
    start=datetime.datetime.now() - datetime.timedelta( seconds= ret)
    print("D... fetching from DATE: ",start)
    if string:
        start=start.strftime("%Y-%m-%d %H:%M:%S")
    return start







#==================================================================

def mongo2h5( DATABASE="", COLLECTION="",  nlimit=int(1e+9), tfrom="100y",  initialconfig="~/.mymongo.local", write_h5=True):
    """
    Creates h5 FILE. NEEDS: database collection ( test,test )
    """
    #
    if DATABASE=="":
            get_mongo_database( DATABASE )
            print("________________ list of databases _______")
            quit()
    #
    #
    if COLLECTION=="":
            #get_mongo_database( DATABASE )
            tell_mongo_collections( DATABASE )
            print("________________ list of collections _______")
            quit()
    #
    db=get_mongo_database( DATABASE , initialconfig , return_none=False)
    collection=db[ COLLECTION ]
    # i have db and collection
    # i sort by time  t
    collection.ensure_index(  [ ("t", pymongo.ASCENDING) ]   )
    READ_CONDITION={}
    start=get_date_from_mark( "100y", string=False ) # 1d 1h 1w
    READ_CONDITION['t']={'$gte': start}
    print("D.... READ_CONDITION", READ_CONDITION," nlimit=", nlimit)
    #start
    #     READ_CONDITION['t']={'$gte': start}
    #==== ii in arg.values ===
    #
    #READ_CONDITION[ ]={'$ne':None }
    #
    cursor=collection.find(  READ_CONDITION  ).sort([('t',-1)]).limit( nlimit )
    licursor=list(cursor)
    if len(licursor)==0:
        print("X... NO DATA")
        quit()
    print("D...  creating DataFrame")
    df=pd.DataFrame( licursor )
    print("D...  deleting _id")
    del df["_id"]               # DELETE _id, dont mess with it
    cols = df.columns.tolist()
    #---------------put 't' in front
    print("D...  reorganizing DataFrame")
    if 't' in cols:
        while cols[0]!='t':
            cols=cols[-1:]+cols[:-1]
    else:
        if DEBUG:print("D...  NO TIME variable.... 't' expected")
    df=df[cols]
    print(df)
    #with pd.option_context('display.max_rows', None, 'display.max_columns', None):  # more options can be specified also
    #    print(df)
    if not write_h5: return
    name=DATABASE+"_"+COLLECTION+"_"+datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
    hdf = pd.HDFStore( name+".h5" )
    hdf.put( name  , df,  format='table',  data_columns=True)
    ks= hdf.keys()
    for i in ks:
        print( "  ", i, hdf[i].shape)









#=================================================


#  create DICT with floats from programmed arguments
#def prepare_write( **kwargs ):  # a=1,b=2....
def prepare_write_no_time( kwargs , silent=True):      # DICT
    """
    INTERNAL: help for prepare_write_no_time  ['t'] is in input
    """
    tnew=datetime.datetime.strptime(kwargs['t'],"%Y-%m-%d %H:%M:%S")
    val={"t":tnew }
    for k in kwargs.keys():
        if not silent:print("    k:w=",k,":",kwargs[k] )
        if kwargs[k] is None:
            val[ k ] = None
        elif k!="t":
            val[ k ] = float( kwargs[k] )

    if not silent:print("FINAL",val)
    return val







#=================================================

######   NORMAL -  WITH AUTOMATIC TIME ########
def prepare_write( kwargs ):      # DICT
    """
    INTERNAL: help for prepare_wite - gets current time
    """
    val={ "t":datetime.datetime.now()
    }
    for k in kwargs.keys():
        print("    k:w=",k,":",kwargs[k] )
        if kwargs[k] is None:
            val[ k ] = None
        else:
            val[ k ] = float( kwargs[k] )
    print("FINAL",val)
    return val







#=================================================

def mongowrite( DATABASE="test", COLLECTION="test",
                values="",
                initialconfig="~/.mymongo.local" , influx_too=False ):
    """
    Writes into MONGO  database (test) collection (test) values (a=1,b=2)
    """
    #
    if DATABASE=="":
            get_mongo_database( DATABASE )
            print("________________ list of databases _______")
            quit()
    #
    #
    if COLLECTION=="":
            #get_mongo_database( DATABASE )
            tell_mongo_collections( DATABASE )
            print("________________ list of collections _______")
            quit()
    #
    #
    #
    db=get_mongo_database( DATABASE , initialconfig , return_none=False)
    collection=db[ COLLECTION ]
    #
    #
    #========= prepare insert values ---------------- make THEM DICT
    INSERTDICT={}
    if DEBUG: print("D...   preapring write - general")
    #insertvals=",".join(values)
    if len(values)==0:
        print("D....  no values given ... calling mongo2h5")
        mongo2h5( DATABASE, COLLECTION, nlimit=100000, write_h5=False)
        quit()

    #
    #
    #
    #
    #=================== WRITe TO INFLUX ================
    if not influx_too:
        influx.influxwrite( DATABASE, COLLECTION, values)
    #
    #
    #
    insertvals=values.split(",")
    print("D... insert pairs:", insertvals)
    for i in insertvals:
        print("D... insert pair:",i)
        key=i.split("=")[0]
        val=i.split("=")[1]
        INSERTDICT[key]=val
    if DEBUG:print("D... INSERT DICT:", INSERTDICT )
    #_____________________________________________________
    #
    #
    #    ###################### WRITE SECTION #################
    if "t" in INSERTDICT:  # key in dict
        if DEBUG:print("D...  time insertion detected...")
        val=prepare_write_no_time( INSERTDICT )
    else:
        if DEBUG:print("D... automatic time...")
        val=prepare_write( INSERTDICT )
    print("i... INSERTING to {} / {} <=".format(DATABASE,COLLECTION), INSERTDICT)
    result=collection.insert_one( val )
    print("result ID of write:", result.inserted_id )







#=================================================
def help( arg="" ):
    """help - not used
    """
    print("H... help")
    if len(arg)>0:
        Fire(arg)
#=================================================





#=============================================================
#=============================================================
#=============================================================
if __name__=="__main__":
    print("D................................................")
    print("D...    database (test,data) - collections - values?")
    print("D...  ./mongo.py  read_mongo_credentials       ")
    print("D...  ./mongo.py  get_mongo_database test      ")
    print("D...  ./mongo.py  tell_mongo_collections test  ")
    print("D...  ./mongo.py  mongo2h5 test home_electro")
    print("D...  ")
    print("D...  ./mongo.py  mongowrite ")
    print("D...  ./mongo.py  mongowrite test test test=5")
    print("D... ./mongo.py  mongo2h5 test test ")
    print("D...  ")

    Fire( {
      'read_mongo_credentials': read_mongo_credentials,
      'get_mongo_database': get_mongo_database,
      'tell_mongo_collections': tell_mongo_collections,
#      'get_date_from_mark': get_date_from_mark,
      'mongo2h5':mongo2h5 ,
#      'prepare_write_no_time':prepare_write_no_time ,
#      'prepare_write':prepare_write ,
      'mongowrite':mongowrite ,
#      'help': help
  } )
